"""
This module provides functionality to interact with the Srun authentication system.
Modified from: https://github.com/iskoldt-X/SRUN-authenticator
"""

import json
import requests
import time
import re
import hmac
import hashlib
import math

def get_md5(password, token):
    return hmac.new(token.encode(), password.encode(), hashlib.md5).hexdigest()

def get_sha1(value):
    return hashlib.sha1(value.encode()).hexdigest()

def force(msg):
    ret = []
    for w in msg:
        ret.append(ord(w))
    return bytes(ret)

def ordat(msg, idx):
    if len(msg) > idx:
        return ord(msg[idx])
    return 0

def sencode(msg, key):
    l = len(msg)
    pwd = []
    for i in range(0, l, 4):
        pwd.append(
            ordat(msg, i) | ordat(msg, i + 1) << 8 | ordat(msg, i + 2) << 16
            | ordat(msg, i + 3) << 24)
    if key:
        pwd.append(l)
    return pwd

def lencode(msg, key):
    l = len(msg)
    ll = (l - 1) << 2
    if key:
        m = msg[l - 1]
        if m < ll - 3 or m > ll:
            return
        ll = m
    for i in range(0, l):
        msg[i] = chr(msg[i] & 0xff) + chr(msg[i] >> 8 & 0xff) + chr(
            msg[i] >> 16 & 0xff) + chr(msg[i] >> 24 & 0xff)
    if key:
        return "".join(msg)[0:ll]
    return "".join(msg)

def get_xencode(msg, key):
    if msg == "":
        return ""
    pwd = sencode(msg, True)
    pwdk = sencode(key, False)
    if len(pwdk) < 4:
        pwdk = pwdk + [0] * (4 - len(pwdk))
    n = len(pwd) - 1
    z = pwd[n]
    y = pwd[0]
    c = 0x86014019 | 0x183639A0
    m = 0
    e = 0
    p = 0
    q = math.floor(6 + 52 / (n + 1))
    d = 0
    while 0 < q:
        d = d + c & (0x8CE0D9BF | 0x731F2640)
        e = d >> 2 & 3
        p = 0
        while p < n:
            y = pwd[p + 1]
            m = z >> 5 ^ y << 2
            m = m + ((y >> 3 ^ z << 4) ^ (d ^ y))
            m = m + (pwdk[(p & 3) ^ e] ^ z)
            pwd[p] = pwd[p] + m & (0xEFB8D130 | 0x10472ECF)
            z = pwd[p]
            p = p + 1
        y = pwd[0]
        m = z >> 5 ^ y << 2
        m = m + ((y >> 3 ^ z << 4) ^ (d ^ y))
        m = m + (pwdk[(p & 3) ^ e] ^ z)
        pwd[n] = pwd[n] + m & (0xBB390742 | 0x44C6F8BD)
        z = pwd[n]
        q = q - 1
    return lencode(pwd, False)

class Srun_Py():
    def __init__(self, srun_host='gw.buaa.edu.cn',host_ip='10.200.21.4'):
        self.srun_host = srun_host
        self.init_url = "https://{}".format(srun_host)
        self.get_ip_api = 'https://{}/cgi-bin/rad_user_info?callback=JQuery'.format(srun_host)
        self.get_ip_api_ip= 'https://{}/cgi-bin/rad_user_info?callback=JQuery'.format(host_ip)
        self.get_challenge_api = "https://{}/cgi-bin/get_challenge".format(srun_host)
        self.get_challenge_api_ip = "https://{}/cgi-bin/get_challenge".format(host_ip)
        self.srun_portal_api = "https://{}/cgi-bin/srun_portal".format(srun_host)
        self.srun_portal_api_ip = "https://{}/cgi-bin/srun_portal".format(host_ip)
        self.rad_user_dm_api = "https://{}/cgi-bin/rad_user_dm".format(srun_host)
        self.rad_user_dm_api_ip = "https://{}/cgi-bin/rad_user_dm".format(host_ip)
        self.header = {'Host':srun_host,'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36 Edg/122.0.0.0'}
        self.n = '200'
        self.type = '1'
        self.ac_id = '1'
        self.enc = "srun_bx1"
        self._ALPHA = "LVoJPiCN2R8G90yg+hmFHuacZ1OWMnrsSTXkYpUq/3dlbfKwv6xztjI7DeBE45QA"

    def get_base64(self,s):
        r = []
        x = len(s) % 3
        if x:
            s = s + '\0' * (3 - x)
        for i in range(0, len(s), 3):
            d = s[i:i + 3]
            a = ord(d[0]) << 16 | ord(d[1]) << 8 | ord(d[2])
            r.append(self._ALPHA[a >> 18])
            r.append(self._ALPHA[a >> 12 & 63])
            r.append(self._ALPHA[a >> 6 & 63])
            r.append(self._ALPHA[a & 63])
        if x == 1:
            r[-1] = '='
            r[-2] = '='
        if x == 2:
            r[-1] = '='
        return ''.join(r)

    def get_chksum(self,username,token,hmd5,ip,i):
        chkstr = token + username
        chkstr += token + hmd5
        chkstr += token + self.ac_id
        chkstr += token + ip
        chkstr += token + self.n
        chkstr += token + self.type
        chkstr += token + i
        return chkstr

    def get_info(self,username,password,ip):
        info_temp = {
            "username": username,
            "password": password,
            "ip": ip,
            "acid": self.ac_id,
            "enc_ver": self.enc
        }
        i = re.sub("'", '"', str(info_temp))
        i = re.sub(" ", '', i)
        return i

    def init_getip(self):
        try:
            res = requests.get(self.get_ip_api)
        except:
            try:
                res = requests.get(self.get_ip_api_ip,headers=self.header,verify=False)
            except:
                try:
                    res = requests.get(self.get_ip_api.replace('https','http',1))
                except:
                    res = requests.get(self.get_ip_api_ip.replace('https','http',1),headers=self.header,verify=False)
        data = json.loads(res.text[res.text.find('(')+1:-1])
        ip = data.get('client_ip') or data.get('online_ip')
        username = data.get('user_name')
        return ip, username

    def get_token(self,username,ip):
        get_challenge_params = {
            "callback": "jQuery112404953340710317169_" + str(int(time.time() * 1000)),
            "username": username,
            "ip": ip,
            "_": int(time.time() * 1000),
        }
        test = requests.Session()
        try:
            get_challenge_res = test.get(self.get_challenge_api, params=get_challenge_params, headers=self.header)
        except:
            try:
                get_challenge_res = test.get(self.get_challenge_api_ip, params=get_challenge_params, headers=self.header,verify=False)
            except:
                try:
                    get_challenge_res = test.get(self.get_challenge_api.replace('https','http',1), params=get_challenge_params, headers=self.header)
                except:
                    get_challenge_res = test.get(self.get_challenge_api_ip.replace('https','http',1), params=get_challenge_params, headers=self.header,verify=False)
        token = re.search('"challenge":"(.*?)"', get_challenge_res.text).group(1)
        return token

    def is_connected(self):
        try:
            try:
                res = requests.get(self.get_ip_api)
            except:
                try:
                    res = requests.get(self.get_ip_api_ip,headers=self.header,verify=False)
                except:
                    try:
                        res = requests.get(self.get_ip_api.replace('https','http',1))
                    except:
                        res = requests.get(self.get_ip_api_ip.replace('https','http',1),headers=self.header,verify=False)
            data = json.loads(res.text[res.text.find('(')+1:-1])
            if 'error' in data and data['error']=='not_online_error':
                return True, False, data
            else:
                return True, True, data
        except:
            return False, False

    def do_complex_work(self,username,password,ip,token):
        i = self.get_info(username,password,ip)
        i = "{SRBX1}" + self.get_base64(get_xencode(i, token))
        hmd5 = get_md5(password, token)
        chksum = get_sha1(self.get_chksum(username,token,hmd5,ip,i))
        return i, hmd5, chksum

    def login(self,username,password):
        is_available, is_online, _ = self.is_connected()
        if not is_available or is_online:
            raise Exception('You are already online or the network is not available!')
        ip , _ = self.init_getip()
        token = self.get_token(username,ip)
        i, hmd5, chksum = self.do_complex_work(username,password,ip,token)
        srun_portal_params = {
            'callback': 'jQuery11240645308969735664_' + str(int(time.time() * 1000)),
            'action': 'login',
            'username': username,
            'password': '{MD5}' + hmd5,
            'ac_id': self.ac_id,
            'ip': ip,
            'chksum': chksum,
            'info': i,
            'n': self.n,
            'type': self.type,
            'os': 'windows+10',
            'name': 'windows',
            'double_stack': '0',
            '_': int(time.time() * 1000)
        }
        test = requests.Session()
        try:
            srun_portal_res = test.get(self.srun_portal_api, params=srun_portal_params, headers=self.header)
        except:
            try:
                srun_portal_res = test.get(self.srun_portal_api_ip, params=srun_portal_params, headers=self.header,verify=False)
            except:
                try:
                    srun_portal_res = test.get(self.srun_portal_api.replace('https','http',1), params=srun_portal_params, headers=self.header)
                except:
                    srun_portal_res = test.get(self.srun_portal_api_ip.replace('https','http',1), params=srun_portal_params, headers=self.header,verify=False)
        srun_portal_res = srun_portal_res.text
        data = json.loads(srun_portal_res[srun_portal_res.find('(')+1:-1])
        return data.get('error') == 'ok'
    
    def logout(self):
        is_available, is_online, _ = self.is_connected()
        if not is_available or not is_online:
            raise Exception('You are not online or the network is not available!')
        ip , username = self.init_getip()
        t = int(time.time() * 1000);
        sign = get_sha1(str(t) + username + ip + '0' + str(t));
        user_dm_params={
            'ip': ip,
            'username': username,
            'time': t,
            'unbind': 0,
            'sign': sign
        }
        test = requests.Session()
        try:
            user_dm_res = test.get(self.rad_user_dm_api, params=user_dm_params, headers=self.header)
        except:
            try:
                user_dm_res = test.get(self.rad_user_dm_api_ip, params=user_dm_params, headers=self.header,verify=False)
            except:
                try:
                    user_dm_res = test.get(self.rad_user_dm_api.replace('https','http',1), params=user_dm_params, headers=self.header)
                except:
                    user_dm_res = test.get(self.rad_user_dm_api_ip.replace('https','http',1), params=user_dm_params, headers=self.header,verify=False)
        user_dm_res = user_dm_res.text
        return user_dm_res=='logout_ok'



import argparse
import json
from srunpy import SrunClient, MainWindow, TaskbarIcon, GUIBackend, __version__

def Cli():
    parser = argparse.ArgumentParser(description='深澜网关登录器(第三方) 命令行 v'+__version__)
    parser.add_argument('-i', '--info', action="store_true", help='显示网关状态 Info')
    parser.add_argument('-l', '--login', action="store_true", help='登录网关 Login')
    parser.add_argument('-o', '--logout', action="store_true", help='登出网关 Logout')
    parser.add_argument('-u', '--username', default=None, help='登录用户名 Username')
    parser.add_argument('-p', '--passwd', default=None, help='登录密码 Password')
    args = parser.parse_args()
    mode=None
    if args.info:
        mode = 'info'
    elif args.login:
        mode = 'login'
    elif args.logout:
        mode = 'logout'
    if mode is None:
        print('深澜网关登录器(第三方) 命令行 v'+__version__)
        print('SrunClient (Third-party) Command Line v'+__version__)
        print('1. 判断登录状态 Check login status')
        print('2. 登录账号 Login account')
        print('3. 登出账号 Logout account')
        command = input('请输入命令 Enter command:')
        if command == '1':
            mode = 'info'
        elif command == '2':
            mode = 'login'
        elif command == '3':
            mode = 'logout'
        else:
            print('未知操作! Unknown operation!')
    if mode is not None:
        srun_client = SrunClient()
        if mode == 'info':
            is_available, is_online, online_data = srun_client.is_connected()
            print('网络是否可用 Available:', is_available)
            print('是否已登录 Online:', is_online)
            if is_online:
                print('在线信息 Online data:')
                print(json.dumps(online_data, indent=4, ensure_ascii=False))
        elif mode == 'login':
            if args.username is None:
                username = input('请输入用户名 Username:')
            else:
                username = args.username
            if args.passwd is None:
                import getpass
                passwd = getpass.getpass('请输入密码 Password:')
            else:
                passwd = args.passwd
            srun_client.login(username, passwd)
        elif mode == 'logout':
            srun_client.logout()

def Gui():
    parser = argparse.ArgumentParser(description='深澜网关登录器(第三方) 用户界面 v'+__version__)
    parser.add_argument('--no-auto-open', action="store_true", help='不自动打开主界面 Do not open the main window automatically')
    parser.add_argument('--qt', action="store_true", help='使用Qt引擎 Use Qt engine')
    args = parser.parse_args()
    srunpy = GUIBackend(use_qt=args.qt)
    main_window = MainWindow(srunpy, not args.no_auto_open)
    while True:
        TaskbarIcon()
        main_window.start_webview()


PROGRAM_VERSION = (1, 0, 6, 1)
__version__ = '.'.join(map(str, PROGRAM_VERSION))

from .html import WebRoot
from .srun import Srun_Py as SrunClient
from .interface import MainWindow, TaskbarIcon, GUIBackend


print('SrunClient version:', __version__)


#!/usr/bin/python3

from srunpy import SrunClient, PROGRAM_VERSION, WebRoot

import os
import sys
import time
import json
import base64
import socket
import webview
import pystray
import requests
import threading
import webbrowser
import subprocess
from Crypto.Cipher import AES
from binascii import b2a_hex, a2b_hex
from PIL import Image
import win32con
import win32api
import win32com.client as client
from win10toast import ToastNotifier

def is_ip_address(address):
    try:
        socket.inet_aton(address)
        return True
    except socket.error:
        return False

def is_domain(address):
    if not is_ip_address(address):
        try:
            ip=socket.gethostbyname(address)
            return True,ip
        except socket.error:
            return False,""
    return False,""

sysToaster = ToastNotifier()


#qt_backend = False
current_pid = os.getpid()
# resource_path = os.path.dirname(os.path.abspath(__file__))
application_path = os.path.abspath(sys.argv[0])
python_path = os.path.abspath(sys.executable)
start_lnk_path = os.path.join(os.path.expandvars(
    r'%APPDATA%'), r'Microsoft\Windows\Start Menu\Programs\Startup', '校园网登陆器.lnk')
appdata_path = os.path.expandvars(r'%APPDATA%')
config_path = os.path.join(appdata_path, 'SRunPy', 'config.json')

def exit_application():
    os._exit(0)


def webbrowser_open(url):
    webbrowser.open(url)


def get_Color_Mode():
    # coding:utf-8
    reg_root = win32con.HKEY_CURRENT_USER
    reg_path = r'Software\Microsoft\Windows\CurrentVersion\Themes\Personalize'
    reg_flags = win32con.KEY_READ | win32con.KEY_WOW64_64KEY
    # 读取键值
    key = win32api.RegOpenKey(reg_root, reg_path, 0, reg_flags)
    value, _ = win32api.RegQueryValueEx(key, "SystemUsesLightTheme")

    # 关闭键
    win32api.RegCloseKey(key)
    return value

def get_Update():
    try:
        response = requests.get(
            "https://api.github.com/repos/HofNature/SRunPy-GUI/releases/latest")
        if response.status_code == 200:
            data = response.json()
            tag_name = data['tag_name']
            if tag_name[1:] > '.'.join(map(str, PROGRAM_VERSION)):
                return True
        return False
    except:
        return False

def load_config():
    aes = MyAES(key="dj26Dh47useoUI28")
    if not os.path.exists(config_path):
        os.makedirs(os.path.dirname(config_path), exist_ok=True)
        config = {
            "username": "",
            "password": "",
            "pass_correct": False,
            "srun_host": "gw.buaa.edu.cn",
            "self_service": "zfw.buaa.edu.cn",
            "host_ip": "10.200.21.4",
            "sleeptime": 5,
            "auto_login": False,
            "start_with_windows": False,
        }
        with open(config_path, 'w') as f:
            f.write(json.dumps(config, indent=4, ensure_ascii=True))
    else:
        with open(config_path, 'r') as f:
            config = json.load(f)
        if config['password'] != "":
            config['password'] = aes.decode_aes(config['password'].encode())
        if not config.get('pass_correct'):
            config['auto_login'] = False
    return config

def reset_config():
    os.remove(config_path)

def save_config(config):
    aes = MyAES(key="dj26Dh47useoUI28")
    config['password'] = aes.encode_aes(config['password']).decode()
    with open(config_path, 'w') as f:
        f.write(json.dumps(config, indent=4, ensure_ascii=True))
    config['password'] = aes.decode_aes(config['password'].encode())


def check_lnk():
    return os.path.exists(start_lnk_path)


def delete_lnk():
    if check_lnk():
        os.remove(start_lnk_path)

def create_desktop_lnk(qt_backend=False):
    no_cmd_path=os.path.join(os.path.dirname(application_path), 'srunpy-gui.exe')
    if python_path != application_path and os.path.exists(python_path) and os.path.basename(application_path).endswith(".exe") and os.path.exists(no_cmd_path):
        desktop_lnk=os.path.join(os.path.expandvars(r'%USERPROFILE%'), 'Desktop', '校园网登陆器.lnk')
        if os.path.exists(desktop_lnk):
            os.remove(desktop_lnk)
        shell = client.Dispatch('Wscript.Shell')
        link = shell.CreateShortCut(desktop_lnk)
        link.TargetPath = no_cmd_path
        #link.Arguments = ' --no-auto-open'
        if qt_backend:
            link.Arguments += ' --qt'
        link.IconLocation = os.path.join(WebRoot, 'icons/logo.ico')+',0'
        link.save()
    else:
        print("非EntryPoint启动，无法创建桌面快捷方式")

def create_lnk(qt_backend=False):
    delete_lnk()
    shell = client.Dispatch('Wscript.Shell')
    link = shell.CreateShortCut(start_lnk_path)
    no_cmd_path=os.path.join(os.path.dirname(application_path), 'srunpy-gui.exe')
    if python_path == application_path or not os.path.exists(python_path):
        link.TargetPath = application_path
        link.Arguments = ' --no-auto-open'
        link.IconLocation = application_path+',0'
    elif os.path.exists(python_path) and os.path.basename(application_path).endswith(".exe") and os.path.exists(no_cmd_path):
        link.TargetPath = no_cmd_path
        link.Arguments = ' --no-auto-open'
        link.IconLocation = os.path.join(
            WebRoot, 'icons/logo.ico')+',0'
    else:
        link.TargetPath = python_path
        link.Arguments = '"'+application_path+'" --no-auto-open'
        link.IconLocation = os.path.join(
            WebRoot, 'icons/logo.ico')+',0'
    if qt_backend:
        link.Arguments += ' --qt'
    link.save()


class MyAES:
    def __init__(self, key):
        self.key = key.encode()

    def __add_to_16(self, text):
        """ 如果string不足16位则用空格补齐16位 """
        if len(text.encode()) % 16:
            add = 16 - (len(text.encode()) % 16)
        else:
            add = 0
        text += ("\0" * add)
        return text.encode()

    def encode_aes(self, text):
        cryptos = AES.new(key=self.key, mode=AES.MODE_ECB)
        cipher_text = cryptos.encrypt(self.__add_to_16(text))
        # 由于AES加密后的字符串不一定是ascii字符集，所以转为16进制字符串
        return b2a_hex(cipher_text)

    def decode_aes(self, text):
        cryptos = AES.new(key=self.key, mode=AES.MODE_ECB)
        plain_text = cryptos.decrypt(a2b_hex(text))
        return bytes.decode(plain_text).rstrip("\0")


class TaskbarIcon():
    def __init__(self):
        self.menu = pystray.Menu(
            pystray.MenuItem("打开主界面", self.stop, default=True),
            pystray.MenuItem("退出登陆器", self.exit)
        )
        try:
            if get_Color_Mode() == 0:
                icon_path = r'icons\journey_white.png'
            else:
                icon_path = r'icons\journey.png'
        except:
            icon_path = r'icons\logo.png'
        self.icon = pystray.Icon("SRunPy", Image.open(
            os.path.join(WebRoot, icon_path)), "校园网登陆器", self.menu)
        self.icon.run()

    def stop(self):
        self.icon.stop()

    def exit(self):
        self.icon.stop()
        os._exit(0)


class GUIBackend():
    def __init__(self,use_qt=False):
        if use_qt:
            try:
                import webview.platforms.qt
            except ImportError:
                use_qt = False
        self.qt_backend = use_qt
        self.auto_login_thread = None
        self.isUptoDate = False
        self.hasDoneUpdate = False
        def check_update():
            self.isUptoDate = get_Update()
        threading.Thread(target=check_update).start()
        self.refresh_config()
        if 'process_id' in self.config and self.config['process_id'] != current_pid:
            subprocess.call("start /B taskkill /f /pid "+str(self.config['process_id']),shell=True)
        if 'process_id' not in self.config:
            try:
                create_desktop_lnk(qt_backend=self.qt_backend)
            except:
                pass
        self.config["process_id"] = current_pid
        save_config(self.config)

    def refresh_config(self):
        try:
            self.config = load_config()
            self.username = self.config['username']
            self.password = self.config['password']
            self.pass_correct = self.config['pass_correct']
            self.srun_host = self.config['srun_host']
            self.host_ip = self.config['host_ip']
            self.self_service = self.config['self_service']
            self.sleeptime = self.config['sleeptime']
            self.auto_login = self.config['auto_login']
            self.start_with_windows = self.config['start_with_windows']
        except Exception as e:
            print(str(e))
            reset_config()
            self.refresh_config()
            return
        if self.srun_host == "":
            self.srun = SrunClient(self.host_ip, self.host_ip)
        else:
            self.srun = SrunClient(self.srun_host, self.host_ip)
        if self.start_with_windows:
            create_lnk(qt_backend=self.qt_backend)
        else:
            delete_lnk()
        if self.auto_login:
            if self.auto_login_thread is None or not self.auto_login_thread.is_alive():
                self.auto_login_thread = threading.Thread(
                    target=self.auto_login_deamon)
                self.auto_login_thread.start()

    def set_config(self, username, password):
        if username != "" and username != self.config['username']:
            if self.srun_host == "gw.buaa.edu.cn":
                self.config['username'] = username.lower()
            else:
                self.config['username'] = username
            self.pass_correct=False
        if password != "" and password != self.config['password']:
            self.config['password'] = password
            self.pass_correct=False
        save_config(self.config)
        self.refresh_config()

    def set_start_with_windows(self, start_with_windows):
        self.config['start_with_windows'] = start_with_windows
        save_config(self.config)
        self.refresh_config()

    def set_auto_login(self, auto_login):
        if auto_login and not self.auto_login and not self.pass_correct:
            return False
        else:
            self.config['auto_login'] = auto_login
            save_config(self.config)
            self.refresh_config()
            return True

    def get_config(self):
        return self.username, self.password != "", self.auto_login, self.start_with_windows, self.isUptoDate, self.hasDoneUpdate, self.srun_host if self.srun_host != "" else self.host_ip, self.self_service
    
    def do_update(self,open=False):
        if open:
            webbrowser.open("https://github.com/HofNature/SRunPy-GUI/releases/latest")
        self.hasDoneUpdate = True

    def start_self_service(self):
        is_available, is_online, data = self.srun.is_connected()
        if is_online:
            if "user_name" in data:
                username = data["user_name"]
            else:
                username = self.username
            data = base64.standard_b64encode(f"{username}:{username}".encode()).decode()
            webbrowser.open(f"http://{self.self_service}/site/sso?data={data}")
        else:
            webbrowser.open(f"http://{self.self_service}")

    def set_srun_host(self, srun_host, self_service):
        # 判断地址是域名还是ip
        domain, ip = is_domain(srun_host)
        if domain:
            self.config['srun_host'] = srun_host
            self.config['host_ip'] = ip
        elif is_ip_address(srun_host):
            self.config['srun_host'] = ""
            self.config['host_ip'] = srun_host
        else:
            return False
        self.config['self_service'] = self_service
        save_config(self.config)
        self.refresh_config()
        del self.srun
        if self.srun_host == "":
            self.srun = SrunClient(self.host_ip, self.host_ip)
        else:
            self.srun = SrunClient(self.srun_host, self.host_ip)
        return True
    
    def auto_login_deamon(self):
        login_failed_count = 0
        while self.auto_login:
            try:
                is_available, is_online, _ = self.srun.is_connected()
            except:
                is_available, is_online = False, False
            if is_available and not is_online:
                try:
                    if self.login():
                        sysToaster.show_toast(
                            "校园网登陆器", "自动登陆成功", duration=5, threaded=True)
                        login_failed_count = 0
                    else:
                        sysToaster.show_toast(
                            "校园网登陆器", "自动登陆失败", duration=5, threaded=True)
                        login_failed_count += 1
                except:
                    sysToaster.show_toast(
                        "校园网登陆器", "自动登陆失败", duration=5, threaded=True)
                    login_failed_count += 1
            if self.auto_login:
                time.sleep(self.sleeptime)
                if login_failed_count > 3:
                    sysToaster.show_toast(
                        "校园网登陆器", "自动登陆失败次数过多，请检查账号密码", duration=180, threaded=True)
                    time.sleep(60*(login_failed_count-3))
            else:
                break

    def login(self):
        try:
            success = self.srun.login(self.username, self.password)
        except:
            success = False
        if success and not self.pass_correct:
            self.config['pass_correct'] = True
            save_config(self.config)
            self.refresh_config()
        return success

    def logout(self):
        try:
            return self.srun.logout()
        except:
            return False

    def get_online_data(self):
        try:
            return self.srun.is_connected()
        except:
            return False, False, {}


class MainWindow():
    def __init__(self, srunpy, open_window=True):
        self.srunpy = srunpy
        self.window = None
        if self.srunpy.qt_backend:
            self.icon_path = os.path.join(WebRoot, r'icons\logo.png')
        if open_window:
            self.start_webview()

    def start_webview(self):
        if len(webview.windows) > 0:
            print('window exists')
            return
        localization = {
            'global.quitConfirmation': u'确定关闭?',
        }
        self.window = webview.create_window(
            "校园网登陆器", os.path.join(WebRoot, "index.html"), width=400, height=300, resizable=False)
        self.window.expose(self.srunpy.get_online_data, self.srunpy.login, self.srunpy.logout, self.srunpy.set_config,
                           self.srunpy.get_config, webbrowser_open, exit_application, self.srunpy.set_start_with_windows, self.srunpy.set_auto_login, self.srunpy.do_update,self.srunpy.start_self_service,self.srunpy.set_srun_host)
        if self.srunpy.qt_backend:
            webview.start(lambda: self.window.evaluate_js(
                'updateInfo()'), localization=localization, debug=False, gui='qt', icon=self.icon_path)
        else:
            webview.start(lambda: self.window.evaluate_js(
                'updateInfo()'), localization=localization, debug=False)



from setuptools import setup, find_packages

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
long_description = long_description.replace("./Show.png","https://github.com/HofNature/SRunPy-GUI/raw/main/Show.png")
setup(
    name="srunpy",
    version="1.0.6.1",
    author="HofNature",
    description="适用于深澜网关的校园网第三方登录器",
    long_description=long_description,
    long_description_content_type='text/markdown',    
    packages=find_packages(),
    install_requires=["pystray","pywebview","pywin32","requests","urllib3","win10toast","pycryptodome"],
    extras_require={
        'qt': ["pywebview[qt]"],
    },
    python_requires=">=3.7",#, <3.13",
    url="https://github.com/HofNature/SRunPy-GUI",
    license="GPL-3.0",
    keywords=["srun", "srunpy", "srun-client", "srun-gui", "srunpy-gui", "srun-client-gui","network","login","logout","gateway"],
    include_package_data=True,
    entry_points={
        "gui_scripts": [
            "srunpy-gui=srunpy.entry:Gui",
        ],
        "console_scripts": [
            "srunpy=srunpy.entry:Gui",
            "srunpy-cli=srunpy.entry:Cli",
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: Microsoft :: Windows",
    ]
)